Package: marr
Type: Package
Title: Maximum rank reproducibility
Version: 1.21.0
Date: 2021-05-10
Authors@R: 
    c(person("Tusharkanti", "Ghosh",
           email = "tusharkantighosh30@gmail.com",
           role = c("aut", "cre")),
    person("Max", "McGrath",
           role = c("aut")),
    person("Daisy", "Philtron",
           role = c("aut")),
    person("Katerina", "Kechris",
           role = c("aut")),
    person("Debashis", "Ghosh",
           role = c("aut", "cph")))
Imports: Rcpp, SummarizedExperiment, utils, methods, ggplot2, dplyr,
        magrittr, rlang, S4Vectors
LinkingTo: Rcpp
Depends: R (>= 4.0)
Suggests: knitr, rmarkdown, BiocStyle, testthat, covr
VignetteBuilder: knitr
Encoding: UTF-8
License: GPL (>= 3)
Description: marr (Maximum Rank Reproducibility) is a
          nonparametric approach that detects
          reproducible signals using a maximal
          rank statistic for high-dimensional biological data. 
          In this R package, we implement functions that 
          measures the reproducibility of features per sample
          pair and sample pairs per feature in high-dimensional
          biological replicate experiments. The 
          user-friendly plot functions in this package also plot 
          histograms of the reproducibility of
          features per sample pair and sample pairs per feature.
          Furthermore, our approach also allows the users to select
          optimal filtering threshold values for the
          identification of reproducible features and sample pairs
          based on output visualization checks (histograms). This 
          package also provides the subset of data filtered by 
          reproducible features and/or sample pairs.
biocViews: QualityControl, Metabolomics, MassSpectrometry, RNASeq,
        ChIPSeq
BugReports: https://github.com/Ghoshlab/marr/issues
RoxygenNote: 7.1.1
git_url: https://git.bioconductor.org/packages/marr
git_branch: devel
git_last_commit: 9d1d483
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: yes
Packaged: 2025-10-31 22:35:50 UTC; biocbuild
Author: Tusharkanti Ghosh [aut, cre],
  Max McGrath [aut],
  Daisy Philtron [aut],
  Katerina Kechris [aut],
  Debashis Ghosh [aut, cph]
Maintainer: Tusharkanti Ghosh <tusharkantighosh30@gmail.com>

# Generated by roxygen2: do not edit by hand

export(Marr)
export(MarrFilterData)
export(MarrPlotFeatures)
export(MarrPlotSamplepairs)
export(MarrProc)
exportClasses(Marr)
exportMethods(MarrAlpha)
exportMethods(MarrData)
exportMethods(MarrFeatureVars)
exportMethods(MarrFeatures)
exportMethods(MarrFeaturesfiltered)
exportMethods(MarrPFeatures)
exportMethods(MarrPSamplepairs)
exportMethods(MarrSamplepairs)
exportMethods(MarrSamplepairsfiltered)
exportPattern('^[[:alpha:]]+')
import(methods)
importClassesFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(Rcpp,evalCpp)
importFrom(Rcpp,sourceCpp)
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,metadata)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowData)
importFrom(dplyr,filter)
importFrom(dplyr,mutate_if)
importFrom(dplyr,select)
importFrom(ggplot2,aes)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(magrittr,"%>%")
importFrom(methods,setClassUnion)
importFrom(rlang,.data)
importFrom(utils,combn)
importFrom(utils,head)
importFrom(utils,tail)
useDynLib(marr)

[![Travis build
status](https://travis-ci.com/Ghoshlab/Marr.svg?branch=master)](https://travis-ci.com/Ghoshlab/Marr)
[![codecov](https://codecov.io/gh/Ghoshlab/Marr/branch/master/graph/badge.svg?token=K3CDL7MEN2)](https://codecov.io/gh/Ghoshlab/Marr)

marr
====

### `marr`: An R/Bioconductor package for Maximum Rank Reproducibility (marr) for high-dimensional biological data.

`marr` measures the reproducibility of features per sample pair and
sample pairs per feature in high-dimensional biological replicate
experiments.

### Method

The `marr` paper published in *Journal of American Statistical
Association*:

> Philtron, Daisy, et al. “Maximum Rank Reproducibility: A Nonparametric
> Approach to Assessing Reproducibility in Replicate Experiments.”
> Journal of the American Statistical Association 113.523 (2018):
> 1028-1039. <https://doi.org/10.1080/01621459.2017.1397521>

`Paper (in preparation)`

> Ghosh, Tusharkanti, et al. “Reproducibility of Mass Spectrometry based
> Metabolomics Data”

### Installing marr

The R-package **marr** can be installed from GitHub using the R package
[devtools](https://github.com/hadley/devtools):

Use to install the latest version of **marr** from GitHub:

    if (!require("devtools")) install.packages("devtools")
    devtools::install_github("Ghoshlab/marr")
    
It can also be installed using Bioconductor: 

```s
# install BiocManager from CRAN (if not already installed)
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")

# install marr package
BiocManager::install("marr")
```

After installation, the package can be loaded into R.
```s
library(marr)
```
    

### Using marr

The main function in the **marr** package is `Marr()`. The `Marr()`
function needs one required object and three optional objects: (1)
object: a data frame or a matrix or a Summarized Experiment with one
assay object with observations (e.g., metabolites or genes) on the rows
and samples as the columns (e.g. let’s call it `dataSE`). (2)
pSamplepairs (Optional) a threshold value that lies between 0 and 1,
used to assign a feature to be reproducible based on the reproducibility
output of the sample pairs per feature. Default is 0.75. (3) pFeatures
(Optional) a threshold value that lies between 0 and 1, used to assign a
sample pair to be reproducible based on the reproducibility output of
the features per sample pair. Default is 0.75. (4) alpha (Optional)
level of significance to control the False Discovery Rate (FDR). Default
is 0.05.

To run the `Marr()` function,

    MarrOutput <- Marr(object = dataSE, pSamplepairs=0.75,
                       pFeatures=0.75, alpha=0.05)

Individual slots can be extracted using accessor methods:

    MarrSamplepairs(MarrOutput) # extract the distribution of percent
    #reproducible features (column-wise) per sample pair
    
    MarrFeatures(MarrOutput) # extract the distribution of percent
    #reproducible sample pairs (row-wise) per feature
    
    MarrSamplepairsfiltered(MarrOutput) # extract the percent of reproducible
    #features based on a threshold value
    
    MarrFeaturesfiltered(MarrOutput) # extract the percent of reproducible
    #sample pairs based on a threshold value

The percent reproducible sample pairs per feature can be directly
plotted using the `MarrPlotFeatures()` function.

    MarrPlotFeatures(MarrOutput) 

The percent reproducible features per sample pair can be directly
plotted using the `MarrPlotSamplepairs()` function.

    MarrPlotSamplepairs(MarrOutput) 

For more details, see `vignettes`.

Bug reports
===========

Report bugs as issues on the [GitHub repository new
issue](https://github.com/Ghoshlab/marr/issues/new)

Contributors
============

-   [Tusharkanti Ghosh](https://github.com/tghosh30)
-   [Max McGrath]()
-   [Daisy Philtron]()
-   [Katerina Kechris]()
-   [Debashis Ghosh](https://github.com/ghoshd)

## ----include=FALSE, echo=FALSE------------------------------------------------
# date: "`r doc_date()`"
# "`r pkg_ver('BiocStyle')`"
# <style>
#     pre {
#     white-space: pre !important;
#     overflow-y: scroll !important;
#     height: 50vh !important;
#     }
# </style>

## ----echo=FALSE, results="hide", message=FALSE--------------------------------
require(knitr)
opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)

## ----style, echo=FALSE, results='asis'----------------------------------------
BiocStyle::markdown()

## ----echo=FALSE, fig.cap="Reproducible Signal matrix", out.width = '100%'-----
knitr::include_graphics("Marr_schematic.png")

## ----load-lib, message=FALSE--------------------------------------------------
library(marr) 

## ----data-1, message=FALSE, warning=FALSE-------------------------------------
data("msprepCOPD")
msprepCOPD

## ----Marr_output--------------------------------------------------------------
library(marr)
Marr_output<- Marr(msprepCOPD, pSamplepairs =
                   0.75, pFeatures = 0.75, alpha=0.05)
Marr_output
## Head of reproducible sample pairs per metabolite (feature)
head(MarrFeatures(Marr_output))
## Head of reproducible metabolites (features) per sample pair
head(MarrSamplepairs(Marr_output))
## Percent of reproducible sample pairs per metabolite (feature)
##greater than 75%
MarrFeaturesfiltered(Marr_output)
## Percent of reproducible metabolites (features) per sample pair
## greater than 75%
MarrSamplepairsfiltered(Marr_output)

## ----fig.cap="Distribution of reproducible metabolites", plot-Marr-Samplepairs----
MarrPlotSamplepairs(Marr_output) 

## ----fig.cap="Distribution of reproducible sample pairs",plot-Marr-Metabolites----
MarrPlotFeatures(Marr_output) 

## ----byFeatures---------------------------------------------------------------
## Filtering the data by reproducible features
MarrFilterData(Marr_output, by = "features")

## Filtering the data by reproducible sample pairs
MarrFilterData(Marr_output, by = "samplePairs")

## Filtering the data by both features and sample pairs
MarrFilterData(Marr_output, by = "both")

## ----session-info-------------------------------------------------------------
sessionInfo() 

#' @title Filter by Maximum Rank Reproducibility
#'
#' @description Filters \code{Marr} object according to the Maximum Rank 
#' Reproducibility of the features, samples pairs, or both. Features are removed
#' if their reproducibility per sample pair is less than \code{pFeatures}.
#' Samples are removed if their sample pair reproducibility per feature is less
#' than \code{pSamplepairs} for all pairings of that sample and the other
#' samples in the set.
#'
#' @param object a Marr object from \code{Marr}
#' @param by String specifying which reproducibility values to filter by. 
#' Options include "features" to filter features according to their 
#' reproducibility, "samplePairs" to filter samples according to the 
#' reproducibility of sample pairs, or "both" to filter both features and sample
#'  pairs according to their respective reproducibility. Default is "both".
#' 
#' @return A list of \code{data.frame}'s or a \code{SummarizedExperiment}. 
#' If a \code{data.frame} was originally input into the \code{Marr} function,
#'  a list with three elements, \code{filteredData}, 
#' \code{removedSamples}, and \code{removedFeatures}, will be returned. If a
#' \code{SummarizedExperiment} was originally input, output will be a 
#' \code{SummarizedExperiment} with the assay filtered and with two metadata
#' objects, \code{removedSamples} and \code{removedFeatures} added.
#' 
#' @examples
#' data <- matrix(rnorm(2400), nrow=200, ncol=12)
#' data_Marr <- Marr(object = data, pSamplepairs=0.75,
#'                   pFeatures=0.75, alpha=0.05)
#' MarrFilterData(data_Marr, by = "both")
#'
#' @export
MarrFilterData <- function(object, by = c("both", "features", "samplePairs")) {
    by <- match.arg(by)
    
    if (!is(object, "Marr")) {
        stop("object must be class Marr")
    }
    
    if (is.data.frame(object@MarrData)) {
        return <- .dfFilter(object, by)
    }
    
    if (is(object@MarrData, "SummarizedExperiment")) {
        return <- .seFilter(object, by)
    }
    
    return
}

#' @importFrom magrittr %>%
#' @importFrom rlang .data
.dfFilter <- function(object, by) {
    if (by == "both" | by == "features") {
        pFeatures <- object@MarrPFeatures
        featuresToKeep <- object@MarrFeatures$reproducibility > pFeatures*100
    } else {
        featuresToKeep <- rep(TRUE, times = nrow(object@MarrData))
    }
    
    originalSamples <- object@MarrData %>%
        colnames()
    featureVars <- object@MarrFeatureVars
    
    if (by == "both" | by == "samplePairs") {
        pSamplePairs <- object@MarrPSamplepairs
        
        samplePairs <- object@MarrSamplepairs %>%
            filter(.data$reproducibility > pSamplePairs*100) %>%
            select(!.data$reproducibility)
        
        samplesToKeep <- c(originalSamples %in% samplePairs$sampleOne |
                               originalSamples %in% samplePairs$sampleTwo |
                               originalSamples %in% featureVars)
        
    } else {
        samplesToKeep <- rep(TRUE, times = ncol(object@MarrData))
    }
    
    filteredData <- object@MarrData[featuresToKeep, samplesToKeep]
    removedSamples <- object@MarrData[, (!samplesToKeep | 
                                             originalSamples %in% featureVars)]
    removedFeatures <- object@MarrData[!featuresToKeep, ]
    
    if(ncol(filteredData) == length(featureVars) | nrow(filteredData) == 0) { 
        filteredData <- NULL 
    }
    if(ncol(removedSamples) == length(featureVars)) { removedSamples <- NULL }
    if(nrow(removedFeatures) == 0) { removedFeatures <- NULL }
    
    return(list("filteredData" = filteredData,
                "removedSamples" = removedSamples,
                "removedFeatures" = removedFeatures))
}

#' @importFrom dplyr filter
#' @importFrom dplyr select
#' @importFrom magrittr %>%
#' @importFrom rlang .data
#' @importFrom S4Vectors metadata
#' @importFrom S4Vectors metadata<-
.seFilter <- function(object, by) {
    if (by == "both" | by == "features") {
        pFeatures <- object@MarrPFeatures
        featuresToKeep <- object@MarrFeatures$reproducibility > pFeatures*100
    } else {
        featuresToKeep <- rep(TRUE, times = nrow(object@MarrData))
    }
    
    if (by == "both" | by == "samplePairs") {
        pSamplePairs <- object@MarrPSamplepairs
        featureVars <- object@MarrFeatureVars
        
        originalSamples <- object@MarrData %>%
            colnames()
        
        samplePairs <- object@MarrSamplepairs %>%
            filter(.data$reproducibility > pSamplePairs*100) %>%
            select(!.data$reproducibility)
        
        samplesToKeep <- c(originalSamples %in% samplePairs$sampleOne |
                               originalSamples %in% samplePairs$sampleTwo |
                               originalSamples %in% featureVars)
        
    } else {
        samplesToKeep <- rep(TRUE, times = ncol(object@MarrData))
    }
    
    filteredData <- object@MarrData[featuresToKeep, samplesToKeep]
    removedSamples <- object@MarrData[, !samplesToKeep]
    removedFeatures <- object@MarrData[!featuresToKeep, ]
    
    if(ncol(filteredData) == 0 | nrow(filteredData) == 0) { 
        filteredData <- NULL 
    }
    if(ncol(removedSamples) == 0) { removedSamples <- NULL }
    if(nrow(removedFeatures) == 0) { removedFeatures <- NULL }
    
    return <- object@MarrData[featuresToKeep, samplesToKeep]
    metadata(return)$removedSamples <- removedSamples
    metadata(return)$removedFeatures <- removedFeatures
        
    return
}

#' @title Marr
#'
#' @description This function applies an Rcpp-based implementation of a
#' computationally efficient method for assessing reproducibility in
#' high-throughput experiments, called the the Marr procedure. This
#' function also defines the Marr class and constructor.
#'
#' @param object an object which is a \code{matrix} or
#' \code{data.frame} with features (e.g. metabolites or genes) on
#' the rows and samples as the columns. Alternatively,
#' a user can provide a \code{SummarizedExperiment} object
#' and the \code{assay(object)} will be used as input
#' for the Marr procedure.
#' @param pSamplepairs (Optional) a threshold value that lies
#' between 0 and 1, used to assign a feature to be reproducible
#' based on the reproducibility output of the sample pairs per feature.
#' Default is 0.75.
#' @param pFeatures (Optional) a threshold value that lies between
#' 0 and 1, used to assign a sample pair to be reproducible based
#' on the reproducibility output of the features per sample pair.
#' Default is 0.75.
#' @param alpha (Optional) level of significance to control
#' the False Discovery Rate (FDR).
#' Default is 0.05.
#' @param featureVars (Optional) Vector of the columns which identify features.
#' If a `SummarizedExperiment` is used for `data`, row variables will be used.
#' @export
#'
#' @return A object of the class \code{Marr} that
#' contains a numeric vector of the Marr sample pairs in
#' the \code{MarrSamplepairs} slot, a numeric vector of the Marr
#' features in the \code{MarrFeatures} slot, a numeric value of
#' the Marr filtered features in the \code{MarrSamplepairsfiltered} slot,
#' and a numeric value of the Marr filtered sample pairs in the
#' \code{MarrFeaturesfiltered} slot.
#' @details
#' marr (Maximum Rank Reproducibility) is a nonparametric approach,
#' which assesses reproducibility in high-dimensional biological
#' replicate experiments. Although it
#' was originally developed for RNASeq data it can be
#' applied across many different high-dimensional biological data
#' including MassSpectrometry based Metabolomics and ChIPSeq. The Marr
#' procedure uses a maximum rank statistic to identify reproducible signals
#' from noise without making any distributional assumptions of reproducible
#' signals. This procedure can be easily applied to a variety of
#' measurement types since it employs a rank scale.
#'
#' This function computes the distributions of percent reproducible
#' sample pairs (row-wise) per feature and percent reproducible  features
#' (column-wise) per sample pair, respectively. Additionally,
#' it also computes the percent of reproducible
#' sample pairs and features based on a threshold value.
#
#' See the vignette for more details.
#'
#' @aliases Marr
#'
#' @docType methods
#' @name Marr
#' @importFrom SummarizedExperiment assays
#' @importFrom SummarizedExperiment assay
#' @importFrom SummarizedExperiment colData
#' @importFrom SummarizedExperiment rowData
#' @importFrom dplyr select
#' @importFrom utils combn
#' @importFrom magrittr %>%
#' @examples
#' data <- matrix(rnorm(2400), nrow=200, ncol=12)
#' data_Marr <- Marr(object = data, pSamplepairs=0.75,
#'                   pFeatures=0.75, alpha=0.05)
#' data("msprepCOPD")
#' data_Marr_COPD <- Marr(object = msprepCOPD, pSamplepairs=0.75,
#'                        pFeatures=0.75, alpha=0.05)
#'
#' @references
#' Philtron, D., Lyu, Y., Li, Q. and Ghosh, D., 2018.
#' Maximum Rank Reproducibility: A Nonparametric Approach to
#' Assessing Reproducibility in Replicate Experiments.
#' Journal of the American Statistical Association, 113(523),
#' pp.1028-1039.
#'
#' @rdname Marr
#' @export
Marr <- function(object, pSamplepairs = 0.75, pFeatures = 0.75,
            alpha = 0.05, featureVars = NULL) {
    
    originalObject <- object

    if (is.data.frame(object)) {
        if(!is.null(featureVars)) {
            featureColumns <- object %>%
                select(featureVars)
            object <- object %>%
                select(-featureVars) %>%
                as.matrix()
        } else {
            object <- as.matrix(object)
            featureColumns <- NULL
        }
        samplePairNames <- combn(colnames(object), 2)
    } else if (is(object, "SummarizedExperiment")) {
        featureColumns <- rowData(object)
        samplePairNames <- combn(rownames(colData(object)), 2)
        object <- assay(object)
    } else if (is(object, "matrix")) {
        if (!is.null(featureVars)) {
            featureColumns <- as.data.frame(object) %>%
                select(featureVars)
            object <- as.data.frame(object) %>%
                select(-featureVars) %>%
                as.matrix()
        } else {
            featureColumns <- NULL
        }
        originalObject <- as.data.frame(object)
        samplePairNames <- combn(colnames(as.data.frame(object)), 2)
    } else {
        stop("The class of the object must be a matrix,
                data.frame or SummarizedExperiment")
    }
    
    if (any(is.na(object))) {
        stop("Object must not contains NAs.")
    }

    Marrutils <- MarrProc(object, alpha)
    samplepairs <- Marrutils$samplepairs
    features <- Marrutils$features
    filSamplepairs <- (length(which(samplepairs > (pSamplepairs *
                100))) * 100)/choose(dim(object)[2], 2)
    filFeatures <- (length(which(features > (pFeatures *
                100))) * 100)/dim(object)[1]
    
    results <- new("Marr")
    results@MarrSamplepairs <- data.frame(sampleOne = 
                                              samplePairNames[1, ],
                                          sampleTwo = 
                                              samplePairNames[2, ],
                                          reproducibility = samplepairs)
    if(is.null(featureColumns)) {
        results@MarrFeatures <- data.frame(reproducibility = features)
    } else {
        results@MarrFeatures <- data.frame(featureColumns, 
                                           reproducibility = features)
    }
    results@MarrSamplepairsfiltered <- filSamplepairs
    results@MarrFeaturesfiltered <- filFeatures
    results@MarrData <- originalObject
    results@MarrPFeatures <- pFeatures
    results@MarrPSamplepairs <- pSamplepairs
    results@MarrAlpha <- alpha
    results@MarrFeatureVars <- featureVars
    
    return(results)
}

library(testthat)
context("checking the marr package")

library(marr)

test_that("checking dimensions of Marr object", {
        n_features <- 1200
        n_samples <- 20
        set.seed(3456)
        data_test <- matrix(rnorm(n_features * n_samples),
        n_features, n_samples)
        Marr_test <- Marr(object = data_test)

        expect_equal(nrow(MarrSamplepairs(Marr_test)),
        choose(n_samples,2))
        expect_equal(nrow(MarrFeatures(Marr_test)), n_features)
})

test_that("checking Marr data", {
        n_features <- 1200
        n_samples <- 20
        set.seed(3456)
        data_test <- matrix(rnorm(n_features * n_samples),
        n_features, n_samples)
        Marr_test <- Marr(object = data_test)

        expect_equal(MarrSamplepairs(Marr_test)$reproducibility[184], 
                     0.08333333,
                     tolerance = 1e-06)
        expect_equal(MarrFeatures(Marr_test)$reproducibility[879], 0.5263158,
        tolerance = 1e-06)
})

#' @useDynLib marr
#' @importFrom Rcpp evalCpp
#' @exportPattern '^[[:alpha:]]+'
NULL

# .estFDR: an internal function to produce a vector of estimated
# false discovery rats based on a vector of maximum rank statistics
# and a khat.
# INPUTS: maxRank = a vector of maximum rank statistics
#         khat = a value of khat calculated from the argmin of
# .mySS (Rcpp internal function in MarrUtility.cpp)
# OUTPUTS: a vector of estimated FDR values for each potential
# threshold Nhat=1,...,n as described in Philtron et al., 2018

.estFDR = function(khat, maxRank) {
            n = length(maxRank)
            Nhat = (khat + 1):n
            Q.khat = sum(maxRank <= khat)
            RNhat <- .RNhatcpp(khat, maxRank)
            indicateR = as.numeric(RNhat > 0)
            RNhat[indicateR == 0] = 1
            temp = Nhat - khat
            numer = temp * temp
            denom = (n - khat) * RNhat
            FDRNhatkhat = (numer/denom) * indicateR
            return(c(rep(0, khat), FDRNhatkhat))
}

# .MaRR: an internal function that performs the Marr procedure based
#        on a vector of maximum rank statistics.
# INPUTS: maxRank = a vector of maximum rank statistics
#         cutoff = a value between 0 and 1 that provides
#                  the maximum allowed value for pi-hat.
#         alpha = desired level of FDR control
#         khat.to.zero = TRUE/FALSE, whether or not to set k-hat to zero
# for mFDR (marginal FDR) calculation (recommended for very small pi1)
# OUTPUTS: khat = n*pi-hat, discrete estimate of where irreproducible
#                 signals begin
#          Nhat = estimated cut-off for maximum ranks that will control
#                 FDR at level alpha
#          estFdr = the estimated fdr value for each of potential N-hat
#          SS = vector of values for SS loss function evaluated
#               at i/n =0, 1/n, 2/n,..., 1
#          whichSig = vector of indices for maximum ranks declared to
#                     be reproducible
.MaRR = function(maxRank, cutoff = 0.9, alpha = 0.05,
            khat.to.zero = FALSE) {
            maxx = floor(cutoff * length(maxRank))
            mySS = .getSS(maxRank)
            khat = which(mySS[seq_len(maxx)] == min(mySS[seq_len(maxx)])) -
                        1
            if (khat.to.zero == TRUE) {
                        khat = 0
            }
            tempFdr = .estFDR(khat, maxRank)
            Nhat = max(which(tempFdr <= alpha))
            whichSig = which(maxRank <= Nhat)
            return(list(Nhat = Nhat, khat = khat, estFdr = tempFdr,
                        SS = mySS, whichSig = whichSig))
}

#' @title Plot percent reproducible features per sample pair
#' for pairwise replicates from \code{Marr} function.
#'
#' @description This function plots a histogram
#' showing the sample pairs along the y-axis
#' and percent reproducible features per sample pair on
#' the x-axis.
#'
#' @param object a Marr object from \code{Marr}
#' @param xLab label for x-axis.
#' Default is 'Percent reproducible features
#'  per sample pair for pairwise replicates'.
#' @param yLab label for y-axis. Default is 'Sample pair'
#' @return A histogram will be created showing the
#' sample pairs along the y-axis and percent reproducible features
#' per sample pair on the x-axis.
#'
#' @importFrom ggplot2 ggplot aes
#' @importFrom ggplot2 geom_histogram
#' @importFrom ggplot2 xlab ylab
#' @export
#' @examples
#' data <- matrix(rnorm(2400), nrow=200, ncol=12)
#' data_Marr <- Marr(object = data, pSamplepairs=0.75,
#'                   pFeatures=0.75, alpha=0.05)
#' MarrPlotSamplepairs(data_Marr)
#'
MarrPlotSamplepairs <- function(  object,
        xLab = "Percent reproducible features per sample pair",
        yLab = "Sample pair") {
        w = object@MarrSamplepairs$reproducibility
        df11 = data.frame(w)
        ggplot(df11, aes(x = w))+
        geom_histogram(binwidth=2,color = "black",
        fill = "white") + ylab(yLab) + xlab(xLab)
}
#' @title Plot percent reproducible sample pairs per feature
#' for pairwise replicates from \code{Marr} function.
#'
#' @description This function plots a histogram
#' showing the features along the y-axis
#' and percent reproducible sample pairs per feature on
#' the x-axis.
#'
#' @param object a Marr object from \code{Marr}
#' @param xLab label for x-axis.
#' Default is 'Percent reproducible sample pairs
#' per feature for pairwise replicates'.
#' @param yLab label for y-axis. Default is 'Feature'
#' @return A histogram will be created showing the
#' features along the y-axis and percent reproducible sample pairs
#' per feature on the x-axis.
#'
#' @importFrom ggplot2 ggplot aes
#' @importFrom ggplot2 geom_histogram
#' @importFrom ggplot2 xlab ylab
#' @export
#' @examples
#' data <- matrix(rnorm(2400), nrow=200, ncol=12)
#' data_Marr <- Marr(object = data, pSamplepairs=0.75,
#'                   pFeatures=0.75, alpha=0.05)
#' MarrPlotFeatures(data_Marr)
#'
MarrPlotFeatures <- function(  object,
        xLab = "Percent reproducible sample pairs per feature",
        yLab = "Feature") {
        ww = object@MarrFeatures$reproducibility
        df12 = data.frame(ww)
        ggplot(df12, aes(x = ww))+
        geom_histogram(binwidth=2,color = "black",
        fill = "white") + ylab(yLab) + xlab(xLab)
}

#' @title S4 Class union
#' @description Class union allowing \code{MarrData} slot to be a data.frame or
#' Summarized Experiment
#' @name data.frameORSummarizedExperiment-class
#' @importFrom methods setClassUnion
setClassUnion("data.frameORSummarizedExperiment",
              c("data.frame", "SummarizedExperiment")) 

#' @title S4 Class union
#' @description Class union allowing \code{MarrFeatureVars} slot to be a vector
#' or NULL
#' @name vectorORNull-class
#' @importFrom methods setClassUnion
setClassUnion("vectorORNull",
              c("vector", "NULL"))

#' @title the Marr class
#'
#' @description  Objects of this class store
#' needed information to work with a
#' Marr object
#'
#' @slot MarrSamplepairs Marr sample pairs
#' @slot MarrFeatures Marr features
#' @slot MarrSamplepairsfiltered Marr sample pairs post filtering
#' @slot MarrFeaturesfiltered Marr metabolites post filtering
#' @slot MarrData Original data object passed to \code{Marr}
#' @slot MarrPSamplepairs Value of \code{pSamplepairs} argument passed 
#' to \code{Marr}
#' @slot MarrPFeatures Value of \code{pFeatures} argument passed to \code{Marr}
#' @slot MarrAlpha Value of \code{alpha} argument passed to \code{Marr}
#' @slot MarrFeatureVars Value of \code{featureVars} passed to \code{Marr}. NULL
#' if \code{featureVars} was left blank
#'
#' @return \code{MarrSamplepairs} returns the distribution of
#' percent reproducible  features (column-wise) per sample pair,
#' \code{MarrFeatures} returns the  distribution of percent reproducible
#' sample pairs (row-wise) per feature,
#' \code{MarrSamplepairsfiltered} returns the percent of reproducible
#' features based on a threshold value and
#' \code{MarrFeaturesfiltered} returns the percent of reproducible
#' sample pairs based on a threshold value
#'
#' @name Marr-class
#' @import methods
#' @importFrom dplyr mutate_if
#' @importClassesFrom SummarizedExperiment SummarizedExperiment
#' @exportClass Marr
#' @aliases Marr-class
#'
#' @examples
#' data <- matrix(rnorm(2400), nrow=200, ncol=12)
#' data_Marr <- Marr(object = data, pSamplepairs=0.75,
#'                   pFeatures=0.75, alpha=0.05)
#'
setClass(Class = "Marr", 
         slot = list(MarrSamplepairs = "data.frame",
                     MarrFeatures = "data.frame",
                     MarrSamplepairsfiltered = "numeric",
                     MarrFeaturesfiltered = "numeric",
                     MarrData = "data.frameORSummarizedExperiment",
                     MarrPSamplepairs = "numeric",
                     MarrPFeatures = "numeric",
                     MarrAlpha = "numeric",
                     MarrFeatureVars = "vectorORNull"))

#' @importFrom utils head
#' @importFrom utils tail
setMethod("show", "Marr", function(object) {
    
    samplePairs <- MarrSamplepairs(object) %>%
        mutate_if(is.numeric, round, digits = 3)
    features <- MarrFeatures(object) %>%
        mutate_if(is.numeric, round, digits = 3)
    
    cat("Marr: Maximum Rank Reproducibility\n")
    cat(c("MarrSamplepairs (length =", nrow(samplePairs), "):", "\n"))
    print.data.frame(head(samplePairs, n = 3))
    cat("...\n")
    cat(c("MarrFeatures (length =", nrow(features), "):", "\n"))
    print.data.frame(head(features, n = 3))
    cat("...\n")
})

#' Example of processed mass spectrometry dataset
#'
#' Data contains LC-MS metabolite analysis for samples from 20 subjects.
#' and 662 metabolites. The raw data was pre-processed using MSPrep method.
#' The raw data pre- processing include 3 steps- Filtering, Missing Value
#' Imputation and Normalization. Filtering- the metabolites(columns)
#' in the raw data were removed if they were missing more than 80 percent
#' of the samples. Missing Value Imputation- The Bayesian
#' Principal Component Analysis (BPCA) was applied to impute the missing values.
#' Normalization- median normalization was applied to remove unwanted
#' variation appears from various sources in metabolomics studies.
#' The first three columns indicate "Mass" indicating the mass-to-charge ratio,
#' "Retention.Time", and "Compound.Name" for each present metabolite.
#' The remaining columns indicate abundance for each of the 645
#' mass/retention-time combination for each subject combination.
#'
#' @docType data
#' @format SummarizedExperiment assay object containing
#' 645 metabolites (features) of 20 subjects (samples).
#' \describe{
#' \item{Mass}{Mass-to-charge ratio}
#' \item{Retention.Time}{Retention-time}
#' \item{Compound.Name}{Compound name for each mass/retention time
#'   combination}
#' \item{X10062C}{The columns indicate metabolite abundances
#'   found in each subject combination. Each column begins with an
#'   'X', followed by the subject ID.}
#' }
#' @source \url{https://www.metabolomicsworkbench.org/data/
#' DRCCMetadata.php?Mode=Project&ProjectID=PR000438}
#'
#' The raw data is available at the NIH Common Fund's National Metabolomics Data
#' Repository (NMDR) website, the Metabolomics Workbench,
#' https://www.metabolomicsworkbench.org, where it has been assigned Project
#' ID PR000438. The raw data can be accessed directly via it's Project DOI:
#' 10.21228/M8FC7C This work is supported by NIH grant, U2C- DK119886.
#'
#' @references
#' Nichole Reisdorph. Untargeted LC-MS metabolomics analysis of human
#' COPD plasma, HILIC & C18, metabolomics_workbench, V1.
#'
#' Hughes, G., Cruickshank-Quinn, C., Reisdorph, R., Lutz, S.,
#' Petrache, I., Reisdorph, N., Bowler, R. and Kechris, K., 2014.
#' MSPrep—Summarization, normalization and diagnostics for processing
#' of mass spectrometry–based metabolomic data.
#' Bioinformatics, 30(1), pp.133-134.
#' @keywords datasets
#' @examples
#' data(msprepCOPD)
#' @usage data(msprepCOPD)
"msprepCOPD"

#' @title Accessors for the 'MarrSamplepairs' slot of a Marr object.
#'
#' @description Accessors for the 'MarrSamplepairs' slot of
#' a Marr object.
#'
#' @usage
#' \S4method{MarrSamplepairs}{Marr}(object)
#'
#' @docType methods
#' @name MarrSamplepairs
#' @rdname MarrSamplepairs
#' @aliases MarrSamplepairs MarrSamplepairs,Marr-method
#' @param object an object of class \code{Marr}.
#'
#' @return The distribution of percent reproducible features
#' (column-wise) per sample pair after applying the maximum
#' rank reproducibility.
#'
#' @export
#'
#' @examples
#' data <- matrix(rnorm(2400), nrow=200, ncol=12)
#' data_Marr <- Marr(object = data, pSamplepairs=0.75,
#'                   pFeatures=0.75, alpha=0.05)
#' MarrSamplepairs(data_Marr)
#'
setMethod(f = "MarrSamplepairs", signature(object = "Marr"),
            function(object) {
                        return(object@MarrSamplepairs)
            })

#' @title Accessors for the 'MarrFeatures' slot of a Marr object.
#'
#' @description Accessors for the 'MarrFeatures' slot
#' of a Marr object.
#'
#' @usage
#' \S4method{MarrFeatures}{Marr}(object)
#'
#' @docType methods
#' @name MarrFeatures
#' @rdname MarrFeatures
#' @aliases MarrFeatures MarrFeatures,Marr-method
#' @param object an object of class \code{Marr}.
#'
#' @return The distribution of percent reproducible sample pairs
#' (row-wise) per feature after applying the maximum rank
#' reproducibility.
#'
#' @export
#'
#' @examples
#' data <- matrix(rnorm(2400), nrow=200, ncol=12)
#' data_Marr <- Marr(object = data, pSamplepairs=0.75,
#'                   pFeatures=0.75, alpha=0.05)
#' MarrFeatures(data_Marr)
#'
setMethod(f = "MarrFeatures", signature(object = "Marr"),
            function(object) {
                        return(object@MarrFeatures)
            })

#' @title Accessors for the 'MarrSamplepairsfiltered' slot of a Marr object.
#'
#' @description Accessors for the 'MarrSamplepairsfiltered'
#' slot of a Marr object.
#'
#' @usage
#' \S4method{MarrSamplepairsfiltered}{Marr}(object)
#'
#' @docType methods
#' @name MarrSamplepairsfiltered
#' @rdname MarrSamplepairsfiltered
#' @aliases MarrSamplepairsfiltered MarrSamplepairsfiltered,Marr-method
#' @param object an object of class \code{Marr}.
#'
#' @return The percent of reproducible features based on a
#' threshold value after applying maximum rank reproducibility.
#'
#' @export
#'
#' @examples
#' data <- matrix(rnorm(2400), nrow=200, ncol=12)
#' data_Marr <- Marr(object = data, pSamplepairs=0.75,
#'                   pFeatures=0.75, alpha=0.05)
#' MarrSamplepairsfiltered(data_Marr)
#'
setMethod(f = "MarrSamplepairsfiltered", signature(object = "Marr"),
            function(object) {
                        return(object@MarrSamplepairsfiltered)
            })

#' @title Accessors for the 'MarrFeaturesfiltered' slot of a Marr object.
#'
#' @description Accessors for the 'MarrFeaturesfiltered'
#' slot of a Marr object.
#'
#' @usage
#' \S4method{MarrFeaturesfiltered}{Marr}(object)
#'
#' @docType methods
#' @name MarrFeaturesfiltered
#' @rdname MarrFeaturesfiltered
#' @aliases MarrFeaturesfiltered MarrFeaturesfiltered,Marr-method
#' @param object an object of class \code{Marr}.
#'
#' @return  The percent of reproducible sample pairs based on a
#' threshold value after applying maximum rank reproducibility.
#'
#' @export
#'
#' @examples
#' data <- matrix(rnorm(2400), nrow=200, ncol=12)
#' data_Marr <- Marr(object = data, pSamplepairs=0.75,
#'                   pFeatures=0.75, alpha=0.05)
#' MarrFeaturesfiltered(data_Marr)
#'
setMethod(f = "MarrFeaturesfiltered", signature(object = "Marr"),
            function(object) {
                        return(object@MarrFeaturesfiltered)
            })

#' @title Accessors for the 'MarrData' slot of a Marr object.
#'
#' @description Accessors for the 'MarrData'
#' slot of a Marr object.
#'
#' @usage
#' \S4method{MarrData}{Marr}(object)
#'
#' @docType methods
#' @name MarrData
#' @rdname MarrData
#' @aliases MarrData MarrData,Marr-method
#' @param object an object of class \code{Marr}.
#'
#' @return  Original data object passed to \code{Marr}
#'
#' @export
#'
#' @examples
#' data <- matrix(rnorm(2400), nrow=200, ncol=12)
#' data_Marr <- Marr(object = data, pSamplepairs=0.75,
#'                   pFeatures=0.75, alpha=0.05)
#' MarrData(data_Marr)
#'
setMethod(f = "MarrData", signature(object = "Marr"),
          function(object) {
              return(object@MarrData)
          })

#' @title Accessors for the 'MarrPSamplepairs' slot of a Marr object.
#'
#' @description Accessors for the 'MarrPSamplepairs'
#' slot of a Marr object.
#'
#' @usage
#' \S4method{MarrPSamplepairs}{Marr}(object)
#'
#' @docType methods
#' @name MarrPSamplepairs
#' @rdname MarrPSamplepairs
#' @aliases MarrPSamplepairs MarrPSamplepairs,Marr-method
#' @param object an object of class \code{Marr}.
#'
#' @return  Value of \code{pSamplepairs} argument passed 
#' to \code{Marr}
#'
#' @export
#'
#' @examples
#' data <- matrix(rnorm(2400), nrow=200, ncol=12)
#' data_Marr <- Marr(object = data, pSamplepairs=0.75,
#'                   pFeatures=0.75, alpha=0.05)
#' MarrPSamplepairs(data_Marr)
#'
setMethod(f = "MarrPSamplepairs", signature(object = "Marr"),
          function(object) {
              return(object@MarrPSamplepairs)
          })

#' @title Accessors for the 'MarrPFeatures' slot of a Marr object.
#'
#' @description Accessors for the 'MarrPFeatures'
#' slot of a Marr object.
#'
#' @usage
#' \S4method{MarrPFeatures}{Marr}(object)
#'
#' @docType methods
#' @name MarrPFeatures
#' @rdname MarrPFeatures
#' @aliases MarrPFeatures MarrPFeatures,Marr-method
#' @param object an object of class \code{Marr}.
#'
#' @return  Value of \code{MarrPFeatures} argument passed 
#' to \code{Marr}
#'
#' @export
#'
#' @examples
#' data <- matrix(rnorm(2400), nrow=200, ncol=12)
#' data_Marr <- Marr(object = data, pSamplepairs=0.75,
#'                   pFeatures=0.75, alpha=0.05)
#' MarrPFeatures(data_Marr)
#'
setMethod(f = "MarrPFeatures", signature(object = "Marr"),
          function(object) {
              return(object@MarrPFeatures)
          })

#' @title Accessors for the 'MarrAlpha' slot of a Marr object.
#'
#' @description Accessors for the 'MarrAlpha'
#' slot of a Marr object.
#'
#' @usage
#' \S4method{MarrAlpha}{Marr}(object)
#'
#' @docType methods
#' @name MarrAlpha
#' @rdname MarrAlpha
#' @aliases MarrAlpha MarrAlpha,Marr-method
#' @param object an object of class \code{Marr}.
#'
#' @return  Value of \code{alpha} argument passed to \code{Marr}
#'
#' @export
#'
#' @examples
#' data <- matrix(rnorm(2400), nrow=200, ncol=12)
#' data_Marr <- Marr(object = data, pSamplepairs=0.75,
#'                   pFeatures=0.75, alpha=0.05)
#' MarrAlpha(data_Marr)
#'
setMethod(f = "MarrAlpha", signature(object = "Marr"),
          function(object) {
              return(object@MarrAlpha)
          })

#' @title Accessors for the 'MarrFeatureVars' slot of a Marr object.
#'
#' @description Accessors for the 'MarrFeatureVars'
#' slot of a Marr object.
#'
#' @usage
#' \S4method{MarrFeatureVars}{Marr}(object)
#'
#' @docType methods
#' @name MarrFeatureVars
#' @rdname MarrFeatureVars
#' @aliases MarrFeatureVars MarrFeatureVars,Marr-method
#' @param object an object of class \code{Marr}.
#'
#' @return Value of \code{featureVars} passed to \code{Marr}. NULL
#' if \code{featureVars} was left blank
#'
#' @export
#'
#' @examples
#' data <- matrix(rnorm(2400), nrow=200, ncol=12)
#' data_Marr <- Marr(object = data, pSamplepairs=0.75,
#'                   pFeatures=0.75, alpha=0.05)
#' MarrFeatureVars(data_Marr)
#'
setMethod(f = "MarrFeatureVars", signature(object = "Marr"),
          function(object) {
              return(object@MarrFeatureVars)
          })

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

.seqen <- function(n) {
    .Call('_marr_seqen', PACKAGE = 'marr', n)
}

.reptest2 <- function(x, y) {
    .Call('_marr_reptest2', PACKAGE = 'marr', x, y)
}

.cumsumSS <- function(x) {
    .Call('_marr_cumsumSS', PACKAGE = 'marr', x)
}

.subset_range1 <- function(x, start, end) {
    .Call('_marr_subset_range1', PACKAGE = 'marr', x, start, end)
}

.ssapplySSS <- function(xx) {
    .Call('_marr_sapplySSS', PACKAGE = 'marr', xx)
}

.getSS <- function(maxRank) {
    .Call('_marr_getSS', PACKAGE = 'marr', maxRank)
}

.seqenkhat <- function(n, k) {
    .Call('_marr_seqenkhat', PACKAGE = 'marr', n, k)
}

.sapplykhat <- function(khat, maxRank) {
    .Call('_marr_sapplykhat', PACKAGE = 'marr', khat, maxRank)
}

.RNhatcpp <- function(khat, maxRank) {
    .Call('_marr_RNhatcpp', PACKAGE = 'marr', khat, maxRank)
}

#' Generic function that returns the Marr sample pairs
#'
#' Given a Marr object, this function returns the
#' Marr sample pairs
#' @rdname MarrSamplepairs
setGeneric(name = "MarrSamplepairs", def = function(object) {
            standardGeneric("MarrSamplepairs")
})

#' Generic function that returns the Marr features
#'
#' Given a Marr object, this function returns the
#' Marr features
#' @rdname MarrFeatures
setGeneric(name = "MarrFeatures", def = function(object) {
            standardGeneric("MarrFeatures")
})

#' Generic function that returns the Marr filtered sample pairs
#'
#' Given a Marr object, this function returns the
#' Marr filtered sample pairs
#' @rdname MarrSamplepairsfiltered
setGeneric(name = "MarrSamplepairsfiltered", def = function(object) {
            standardGeneric("MarrSamplepairsfiltered")
})

#' Generic function that returns the Marr filtered features
#'
#' Given a Marr object, this function returns the
#' Marr filtered features
#' @rdname MarrFeaturesfiltered
setGeneric(name = "MarrFeaturesfiltered", def = function(object) {
            standardGeneric("MarrFeaturesfiltered")
})

#' Generic function that returns the Marr Data
#'
#' Given a Marr object, this function returns the
#' Marr Data
#' @rdname MarrData
setGeneric(name = "MarrData", def = function(object) {
    standardGeneric("MarrData")
})

#' Generic function that returns the Marr P Sample Pairs
#'
#' Given a Marr object, this function returns the
#' Marr P Sample Pairs
#' @rdname MarrPSamplepairs
setGeneric(name = "MarrPSamplepairs", def = function(object) {
    standardGeneric("MarrPSamplepairs")
})

#' Generic function that returns the Marr P Features
#'
#' Given a Marr object, this function returns the
#' Marr P Features
#' @rdname MarrPFeatures
setGeneric(name = "MarrPFeatures", def = function(object) {
    standardGeneric("MarrPFeatures")
})

#' Generic function that returns the Marr Alpha
#'
#' Given a Marr object, this function returns the
#' Marr Alpha
#' @rdname MarrAlpha
setGeneric(name = "MarrAlpha", def = function(object) {
    standardGeneric("MarrAlpha")
})

#' Generic function that returns the Marr Feature Vars
#'
#' Given a Marr object, this function returns the
#' Marr Feature Vars
#' @rdname MarrFeatureVars
setGeneric(name = "MarrFeatureVars", def = function(object) {
    standardGeneric("MarrFeatureVars")
})

#' @title MarrProc
#'
#' @description This function is a helper function that
#' computes distributions of reproducible sample pairs per feature
#' and reproducible features per sample pair for the function
#' \code{Marr}.
#'
#' @param object an object which is a \code{matrix} or
#' \code{data.frame} with features (e.g. metabolites or genes) on
#' the rows and samples as the columns. Alternatively,
#' a user can provide a \code{SummarizedExperiment} object
#' and the \code{assay(object)} will be used as input
#' for the Marr procedure.
#' @param alpha (Optional) level of significance to control
#' the False Discovery Rate (FDR).
#' Default is 0.05.
#'
#' @return A list of percent reproducible statistics including
#' \item{samplepairs}{the distribution of
#' percent reproducible  features (column-wise) per sample pair}
#' \item{features}{the distribution of
#' percent reproducible  sample pairs (row-wise) per feature}
#'
#' @aliases MarrProc
#'
#' @docType methods
#' @examples
#' data <- matrix(rnorm(2400), nrow=200, ncol=12)
#' data_MarrProc <- MarrProc(object=data, alpha = 0.05)
#'
#' @rdname MarrProc
#' @export
MarrProc <- function(object, alpha = 0.05) {
        datranks <- array(0, dim = c(dim(object)[2], dim(object)[1]))
        for (i in seq_len(dim(object)[2])) {
                datranks[i, ] <- rank(-(object[, c(i)]),
                ties.method = "average")
        }
        datMaxRankij <- array(0, dim = c(dim(object)[2],
                                            dim(object)[2], dim(object)[1]))
        rall <- k <- array(0, dim = c(dim(object)[2], dim(object)[2]))
        for (i in seq_len(dim(object)[2])) {
                for (j in i:dim(object)[2]) {
                        datMaxRankij[i, j, ] <- apply(cbind(datranks[i, ],
                        datranks[j,]), 1, max)
                }
        }
        countall = c(0)
        for (i in seq_len(dim(object)[2])) {
                for (j in i:dim(object)[2]) {
                        if (i < j) {
                                listMarr <- .MaRR(datMaxRankij[i, j, ],
                                cutoff = 0.9, alpha, khat.to.zero = FALSE)
                                rall[i, j] <- length(listMarr[[5]])
                                mall_rep_i <- listMarr[[5]]
                                posall <- array(0, dim = c(dim(object)[1]))
                                posall[mall_rep_i] <- rep(1, length(mall_rep_i))
                                countall = countall + (posall == 1) * 1
                        }
                }
        }
        dim1 <- dim(object)[2]
        rall_new <- upper.tri(rall, diag = FALSE)
        rall2 <- rall[rall_new]
        samplepairs <- (rall2 * 100)/dim(object)[1]
        features <- (countall * 100)/(choose(dim1, 2))
        list(samplepairs = samplepairs, features = features)
}

---
title: "The marr user's guide"
author:
- name: Tusharkanti Ghosh
  affiliation: Colorado School of Public Health
- name: Max McGrath
  affiliation: Colorado School of Public Health
- name: Daisy Philtron
  affiliation: Penn State University
- name: Katerina Kechris
  affiliation: Colorado School of Public Health
- name: Debashis Ghosh
  affiliation: Colorado School of Public Health
output: 
  BiocStyle::html_document:
    highlight: "tango"
    code_folding: show
    toc: true
    toc_float: 
      collapsed: false
package: marr
abstract: marr (Maximum Rank Reproducibility) is a
          nonparametric approach that detects
          reproducible signals using a maximal
          rank statistic for high-dimensional biological data.
citation_package: natbib
bibliography: Marr.bib
vignette: |
  %\VignetteIndexEntry{The marr user's guide}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}  
---

```{r, include=FALSE, echo=FALSE}
# date: "`r doc_date()`"
# "`r pkg_ver('BiocStyle')`"
# <style>
#     pre {
#     white-space: pre !important;
#     overflow-y: scroll !important;
#     height: 50vh !important;
#     }
# </style>
```
---


```{r, echo=FALSE, results="hide", message=FALSE}
require(knitr)
opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

```{r style, echo=FALSE, results='asis'}
BiocStyle::markdown()
```


# Introduction

Reproducibility is an on-going challenge with high-throughput technologies that
have been developed in the last two decades for quantifying a wide range of
biological processes. One of the main difficulties faced by researchers is the
variability of output across replicate experiments (@li2011measuring). Several
authors have addressed the issue of reproducibility among high-throughput
experiments (@porazinska2010reproducibility, @marioni2008rna,
@ac2013reproducibility). In each high-throughput experiment (e.g., arrays,
sequencing, mass spectrometry), a large number of features are measured
simultaneously, and candidates are often subjected for follow-up statistical
analysis. We use the term features to refer to biological features (e.g.,
metabolites, genes) resulting from a high-throughput experiment in the rest of
this article.  When measurements show consistency across replicate experiments,
we define that measurement to be reproducible. Similarly, measurements that are
not consistent across replicates may be problematic and should be identified. In
this vignette, features that show consistency across high-dimensional replicate
experiments are termed reproducible and the ones that are not consistent are
termed irreproducible.   The reproducibility of a high-throughput experiment
primarily depends on the technical variables, such as run time, technical
replicates, laboratory operators and biological variables, such as healthy and
diseased subjects. A critical step toward making optimal design choices is to
assess how these biological and technical variables affect reproducibility
across replicate experiments (@talloen2010filtering, 
@arvidsson2008quantprime).

In this vignette, we introduce the marr procedure @Philtron2018, referred to as
**maximum rank reproducibility** (**marr**) to identify reproducible features in
high-throughput replicate experiments. In this vignette, we demonstrate with an 
example data set that the (ma)ximum (r)ank (r)eproducibility (marr) procedure
can be adapted to high-throughput MS-Metabolomics experiments across
(biological or technical) replicate samples
(Ghosh et al, 2020, in preparation).

The marr procedure was originally proposed to assess reproducibility of gene
ranks in replicate experiments. The `marr` R-package contains the `Marr()`
function, which calculates  a matrix of signals ($\text{irreproducible}=0$,
$\text{reproducible}=1$) with $M$ rows (total number of features) and $J$
columns ($J={I \choose 2}$) (replicate sample pairs ${I \choose 2}$), where
$J$ is the total possible number of sample pairs of replicate experiments.
We assign feature $m$ to be reproducible if a certain percentage signals
($100c_s\%$) are reproducible for pairwise combinations of replicate
experiments, i.e.,
if $$ \frac{{\sum_{i<i'}{{{r_{m,{(i,i')}}}}}}}{J} >c_s, $$

such that, $c_s \in (0,1)$.

Similarly, we assign a sample pair $(i,~i')$ to be reproducible if a certain
percentage signals ($100c_m\%$) are reproducible across all features, i.e.,
if $$ \frac{\sum_{m}{{r{_{m,(i,i')}}}}}{M}>c_m, $$
such that, $c_m \in (0,1)$.

The reproducible signal matrix is shown in Figure 1 below.
```{r, echo=FALSE, fig.cap="Reproducible Signal matrix", out.width = '100%'}
knitr::include_graphics("Marr_schematic.png")
```
# Getting Started

Load the package in R
```{r load-lib, message=FALSE}
library(marr) 
```

# msprepCOPD Data

The **marr** package contains a pre-processed data `SummarizedExperiment` assay 
object of 645 metabolites (features) measured in plasma and 20 biological
replicates from the multi-center Genetic Epidemiology of COPD (COPDGene) study
which was designed to study the underlying genetic factors of COPD,
(@Regan2010). We only used a subset of the original raw COPD data in this
vignette.

## msprepCOPD data pre-processing

The **msprepCOPD** data  in the **marr** package was pre-processed using the
MSPrep software (@Hughes2013). The data pre-processing include $3$ steps and
they are as follows:
1. `Filtering`: Metabolites are removed if they are missing more
than $80\%$ of the samples, (@Bijlsma2006, @chong2018metaboanalyst).
Originally, there were 662 metabolites in the raw data. After filtering,
645 metabolites remain.
2. `Missing value imputation technique`: We apply Bayesian Principal Component
Analysis (BPCA) to impute missing values (@hastie1999imputing).
3. `Normalization`: Median normalization are performed.

```{r data-1, message=FALSE, warning=FALSE}
data("msprepCOPD")
msprepCOPD
```

# Using the `Marr()` function

## Input for `Marr()`
The `Marr()` function must have one object as input:
1. `object`: a data frame or a matrix or a SummarizedExperiment object with
abundance measurements of metabolites (features) on the rows and replicates
(samples) as the columns. `Marr()` accepts objects which are a data frame or
matrix with observations (e.g. metabolites) on the rows and replicates as the
columns.
2. `pSamplepairs`: **optional** We assign a metabolite (feature) for a replicate
sample pair to be reproducible using a threshold value of `pSamplepairs`
($c_s=0.75$).
3. `pFeatures`: **optional** We assign a sample pair for a metabolite (feature)
to be reproducible using a threshold value of `pFeatures` ($c_m=0.75$).
4. `alpha`: **optional** level of significance to control the False Discovery
rate (FDR). Default is $0.05$ (i.e., $\alpha=0.05$).

## Running `Marr()`

### msprepCOPD SummarizedExperiment example - Evaluating reproducibility

We apply the Marr procedure to assess the reproducibility of replicates in the
msprepCOPD data. The distribution of reproducible pairs and metabolites
(features) are illustrated in Figures 2 and 3, respectively.
To run the `Marr()` function, we only input the data object. We obtain 4 outputs
after running the `Marr()` function. They are shown below:

```{r Marr_output}
library(marr)
Marr_output<- Marr(msprepCOPD, pSamplepairs =
                   0.75, pFeatures = 0.75, alpha=0.05)
Marr_output
## Head of reproducible sample pairs per metabolite (feature)
head(MarrFeatures(Marr_output))
## Head of reproducible metabolites (features) per sample pair
head(MarrSamplepairs(Marr_output))
## Percent of reproducible sample pairs per metabolite (feature)
##greater than 75%
MarrFeaturesfiltered(Marr_output)
## Percent of reproducible metabolites (features) per sample pair
## greater than 75%
MarrSamplepairsfiltered(Marr_output)
```
The distribution of reproducible metabolites/features (sample pairs) per sample
pair (metabolite) can be extracted using the` MarrSamplepairs()` 
(`MarrFeatures()`) function (see above). The distribution of reproducible
metabolites/features  and sample pairs can plotted using the
`MarrPlotSamplepairs()` and `MarrPlotFeatures()` functions, respectively (see
below).

```{r fig.cap="Distribution of reproducible metabolites", plot-Marr-Samplepairs}
MarrPlotSamplepairs(Marr_output) 
```

```{r fig.cap="Distribution of reproducible sample pairs",plot-Marr-Metabolites}
MarrPlotFeatures(Marr_output) 
```

Figure 2 illustrates percentage of
reproducible metabolites (features) per sample pair in the
$x$-axis.  In Figure 2, the higher percentage of reproducible metabolites
(features) per sample pair in the $x$-axis would indicate
stronger reproducibility between the sample pairs.

Figure 3 illustrates percentage of reproducible sample pairs
per metabolite (feature) in the $x$-axis.  In Figure 3, the higher
percentage of reproducible sample pairs per metabolite
(feature) in the $x$-axis would indicate stronger reproducibility
of a metabolite (feature) across all sample pairs.

## Filtering the data by reproducible features and/or sample pairs
```{r byFeatures}
## Filtering the data by reproducible features
MarrFilterData(Marr_output, by = "features")

## Filtering the data by reproducible sample pairs
MarrFilterData(Marr_output, by = "samplePairs")

## Filtering the data by both features and sample pairs
MarrFilterData(Marr_output, by = "both")
```

# Session Info
```{r session-info}
sessionInfo() 
```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Marr.R
\docType{methods}
\name{Marr}
\alias{Marr}
\title{Marr}
\usage{
Marr(
  object,
  pSamplepairs = 0.75,
  pFeatures = 0.75,
  alpha = 0.05,
  featureVars = NULL
)
}
\arguments{
\item{object}{an object which is a \code{matrix} or
\code{data.frame} with features (e.g. metabolites or genes) on
the rows and samples as the columns. Alternatively,
a user can provide a \code{SummarizedExperiment} object
and the \code{assay(object)} will be used as input
for the Marr procedure.}

\item{pSamplepairs}{(Optional) a threshold value that lies
between 0 and 1, used to assign a feature to be reproducible
based on the reproducibility output of the sample pairs per feature.
Default is 0.75.}

\item{pFeatures}{(Optional) a threshold value that lies between
0 and 1, used to assign a sample pair to be reproducible based
on the reproducibility output of the features per sample pair.
Default is 0.75.}

\item{alpha}{(Optional) level of significance to control
the False Discovery Rate (FDR).
Default is 0.05.}

\item{featureVars}{(Optional) Vector of the columns which identify features.
If a `SummarizedExperiment` is used for `data`, row variables will be used.}
}
\value{
A object of the class \code{Marr} that
contains a numeric vector of the Marr sample pairs in
the \code{MarrSamplepairs} slot, a numeric vector of the Marr
features in the \code{MarrFeatures} slot, a numeric value of
the Marr filtered features in the \code{MarrSamplepairsfiltered} slot,
and a numeric value of the Marr filtered sample pairs in the
\code{MarrFeaturesfiltered} slot.
}
\description{
This function applies an Rcpp-based implementation of a
computationally efficient method for assessing reproducibility in
high-throughput experiments, called the the Marr procedure. This
function also defines the Marr class and constructor.
}
\details{
marr (Maximum Rank Reproducibility) is a nonparametric approach,
which assesses reproducibility in high-dimensional biological
replicate experiments. Although it
was originally developed for RNASeq data it can be
applied across many different high-dimensional biological data
including MassSpectrometry based Metabolomics and ChIPSeq. The Marr
procedure uses a maximum rank statistic to identify reproducible signals
from noise without making any distributional assumptions of reproducible
signals. This procedure can be easily applied to a variety of
measurement types since it employs a rank scale.

This function computes the distributions of percent reproducible
sample pairs (row-wise) per feature and percent reproducible  features
(column-wise) per sample pair, respectively. Additionally,
it also computes the percent of reproducible
sample pairs and features based on a threshold value.
See the vignette for more details.
}
\examples{
data <- matrix(rnorm(2400), nrow=200, ncol=12)
data_Marr <- Marr(object = data, pSamplepairs=0.75,
                  pFeatures=0.75, alpha=0.05)
data("msprepCOPD")
data_Marr_COPD <- Marr(object = msprepCOPD, pSamplepairs=0.75,
                       pFeatures=0.75, alpha=0.05)

}
\references{
Philtron, D., Lyu, Y., Li, Q. and Ghosh, D., 2018.
Maximum Rank Reproducibility: A Nonparametric Approach to
Assessing Reproducibility in Replicate Experiments.
Journal of the American Statistical Association, 113(523),
pp.1028-1039.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MarrProc.R
\docType{methods}
\name{MarrProc}
\alias{MarrProc}
\title{MarrProc}
\usage{
MarrProc(object, alpha = 0.05)
}
\arguments{
\item{object}{an object which is a \code{matrix} or
\code{data.frame} with features (e.g. metabolites or genes) on
the rows and samples as the columns. Alternatively,
a user can provide a \code{SummarizedExperiment} object
and the \code{assay(object)} will be used as input
for the Marr procedure.}

\item{alpha}{(Optional) level of significance to control
the False Discovery Rate (FDR).
Default is 0.05.}
}
\value{
A list of percent reproducible statistics including
\item{samplepairs}{the distribution of
percent reproducible  features (column-wise) per sample pair}
\item{features}{the distribution of
percent reproducible  sample pairs (row-wise) per feature}
}
\description{
This function is a helper function that
computes distributions of reproducible sample pairs per feature
and reproducible features per sample pair for the function
\code{Marr}.
}
\examples{
data <- matrix(rnorm(2400), nrow=200, ncol=12)
data_MarrProc <- MarrProc(object=data, alpha = 0.05)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{vectorORNull-class}
\alias{vectorORNull-class}
\title{S4 Class union}
\description{
Class union allowing \code{MarrFeatureVars} slot to be a vector
or NULL
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/msprepCOPD.R
\docType{data}
\name{msprepCOPD}
\alias{msprepCOPD}
\title{Example of processed mass spectrometry dataset}
\format{
SummarizedExperiment assay object containing
645 metabolites (features) of 20 subjects (samples).
\describe{
\item{Mass}{Mass-to-charge ratio}
\item{Retention.Time}{Retention-time}
\item{Compound.Name}{Compound name for each mass/retention time
  combination}
\item{X10062C}{The columns indicate metabolite abundances
  found in each subject combination. Each column begins with an
  'X', followed by the subject ID.}
}
}
\source{
\url{https://www.metabolomicsworkbench.org/data/
DRCCMetadata.php?Mode=Project&ProjectID=PR000438}

The raw data is available at the NIH Common Fund's National Metabolomics Data
Repository (NMDR) website, the Metabolomics Workbench,
https://www.metabolomicsworkbench.org, where it has been assigned Project
ID PR000438. The raw data can be accessed directly via it's Project DOI:
10.21228/M8FC7C This work is supported by NIH grant, U2C- DK119886.
}
\usage{
data(msprepCOPD)
}
\description{
Data contains LC-MS metabolite analysis for samples from 20 subjects.
and 662 metabolites. The raw data was pre-processed using MSPrep method.
The raw data pre- processing include 3 steps- Filtering, Missing Value
Imputation and Normalization. Filtering- the metabolites(columns)
in the raw data were removed if they were missing more than 80 percent
of the samples. Missing Value Imputation- The Bayesian
Principal Component Analysis (BPCA) was applied to impute the missing values.
Normalization- median normalization was applied to remove unwanted
variation appears from various sources in metabolomics studies.
The first three columns indicate "Mass" indicating the mass-to-charge ratio,
"Retention.Time", and "Compound.Name" for each present metabolite.
The remaining columns indicate abundance for each of the 645
mass/retention-time combination for each subject combination.
}
\examples{
data(msprepCOPD)
}
\references{
Nichole Reisdorph. Untargeted LC-MS metabolomics analysis of human
COPD plasma, HILIC & C18, metabolomics_workbench, V1.

Hughes, G., Cruickshank-Quinn, C., Reisdorph, R., Lutz, S.,
Petrache, I., Reisdorph, N., Bowler, R. and Kechris, K., 2014.
MSPrep—Summarization, normalization and diagnostics for processing
of mass spectrometry–based metabolomic data.
Bioinformatics, 30(1), pp.133-134.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{MarrPlotFeatures}
\alias{MarrPlotFeatures}
\title{Plot percent reproducible sample pairs per feature
for pairwise replicates from \code{Marr} function.}
\usage{
MarrPlotFeatures(
  object,
  xLab = "Percent reproducible sample pairs per feature",
  yLab = "Feature"
)
}
\arguments{
\item{object}{a Marr object from \code{Marr}}

\item{xLab}{label for x-axis.
Default is 'Percent reproducible sample pairs
per feature for pairwise replicates'.}

\item{yLab}{label for y-axis. Default is 'Feature'}
}
\value{
A histogram will be created showing the
features along the y-axis and percent reproducible sample pairs
per feature on the x-axis.
}
\description{
This function plots a histogram
showing the features along the y-axis
and percent reproducible sample pairs per feature on
the x-axis.
}
\examples{
data <- matrix(rnorm(2400), nrow=200, ncol=12)
data_Marr <- Marr(object = data, pSamplepairs=0.75,
                  pFeatures=0.75, alpha=0.05)
MarrPlotFeatures(data_Marr)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filterData.R
\name{MarrFilterData}
\alias{MarrFilterData}
\title{Filter by Maximum Rank Reproducibility}
\usage{
MarrFilterData(object, by = c("both", "features", "samplePairs"))
}
\arguments{
\item{object}{a Marr object from \code{Marr}}

\item{by}{String specifying which reproducibility values to filter by. 
Options include "features" to filter features according to their 
reproducibility, "samplePairs" to filter samples according to the 
reproducibility of sample pairs, or "both" to filter both features and sample
 pairs according to their respective reproducibility. Default is "both".}
}
\value{
A list of \code{data.frame}'s or a \code{SummarizedExperiment}. 
If a \code{data.frame} was originally input into the \code{Marr} function,
 a list with three elements, \code{filteredData}, 
\code{removedSamples}, and \code{removedFeatures}, will be returned. If a
\code{SummarizedExperiment} was originally input, output will be a 
\code{SummarizedExperiment} with the assay filtered and with two metadata
objects, \code{removedSamples} and \code{removedFeatures} added.
}
\description{
Filters \code{Marr} object according to the Maximum Rank 
Reproducibility of the features, samples pairs, or both. Features are removed
if their reproducibility per sample pair is less than \code{pFeatures}.
Samples are removed if their sample pair reproducibility per feature is less
than \code{pSamplepairs} for all pairings of that sample and the other
samples in the set.
}
\examples{
data <- matrix(rnorm(2400), nrow=200, ncol=12)
data_Marr <- Marr(object = data, pSamplepairs=0.75,
                  pFeatures=0.75, alpha=0.05)
MarrFilterData(data_Marr, by = "both")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/AllslotAccessors.R
\docType{methods}
\name{MarrAlpha}
\alias{MarrAlpha}
\alias{MarrAlpha,Marr-method}
\title{Generic function that returns the Marr Alpha}
\usage{
MarrAlpha(object)

\S4method{MarrAlpha}{Marr}(object)
}
\arguments{
\item{object}{an object of class \code{Marr}.}
}
\value{
Value of \code{alpha} argument passed to \code{Marr}
}
\description{
Given a Marr object, this function returns the
Marr Alpha

Accessors for the 'MarrAlpha'
slot of a Marr object.
}
\examples{
data <- matrix(rnorm(2400), nrow=200, ncol=12)
data_Marr <- Marr(object = data, pSamplepairs=0.75,
                  pFeatures=0.75, alpha=0.05)
MarrAlpha(data_Marr)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{Marr-class}
\alias{Marr-class}
\title{the Marr class}
\value{
\code{MarrSamplepairs} returns the distribution of
percent reproducible  features (column-wise) per sample pair,
\code{MarrFeatures} returns the  distribution of percent reproducible
sample pairs (row-wise) per feature,
\code{MarrSamplepairsfiltered} returns the percent of reproducible
features based on a threshold value and
\code{MarrFeaturesfiltered} returns the percent of reproducible
sample pairs based on a threshold value
}
\description{
Objects of this class store
needed information to work with a
Marr object
}
\section{Slots}{

\describe{
\item{\code{MarrSamplepairs}}{Marr sample pairs}

\item{\code{MarrFeatures}}{Marr features}

\item{\code{MarrSamplepairsfiltered}}{Marr sample pairs post filtering}

\item{\code{MarrFeaturesfiltered}}{Marr metabolites post filtering}

\item{\code{MarrData}}{Original data object passed to \code{Marr}}

\item{\code{MarrPSamplepairs}}{Value of \code{pSamplepairs} argument passed 
to \code{Marr}}

\item{\code{MarrPFeatures}}{Value of \code{pFeatures} argument passed to \code{Marr}}

\item{\code{MarrAlpha}}{Value of \code{alpha} argument passed to \code{Marr}}

\item{\code{MarrFeatureVars}}{Value of \code{featureVars} passed to \code{Marr}. NULL
if \code{featureVars} was left blank}
}}

\examples{
data <- matrix(rnorm(2400), nrow=200, ncol=12)
data_Marr <- Marr(object = data, pSamplepairs=0.75,
                  pFeatures=0.75, alpha=0.05)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/AllslotAccessors.R
\docType{methods}
\name{MarrData}
\alias{MarrData}
\alias{MarrData,Marr-method}
\title{Generic function that returns the Marr Data}
\usage{
MarrData(object)

\S4method{MarrData}{Marr}(object)
}
\arguments{
\item{object}{an object of class \code{Marr}.}
}
\value{
Original data object passed to \code{Marr}
}
\description{
Given a Marr object, this function returns the
Marr Data

Accessors for the 'MarrData'
slot of a Marr object.
}
\examples{
data <- matrix(rnorm(2400), nrow=200, ncol=12)
data_Marr <- Marr(object = data, pSamplepairs=0.75,
                  pFeatures=0.75, alpha=0.05)
MarrData(data_Marr)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{MarrPlotSamplepairs}
\alias{MarrPlotSamplepairs}
\title{Plot percent reproducible features per sample pair
for pairwise replicates from \code{Marr} function.}
\usage{
MarrPlotSamplepairs(
  object,
  xLab = "Percent reproducible features per sample pair",
  yLab = "Sample pair"
)
}
\arguments{
\item{object}{a Marr object from \code{Marr}}

\item{xLab}{label for x-axis.
Default is 'Percent reproducible features
 per sample pair for pairwise replicates'.}

\item{yLab}{label for y-axis. Default is 'Sample pair'}
}
\value{
A histogram will be created showing the
sample pairs along the y-axis and percent reproducible features
per sample pair on the x-axis.
}
\description{
This function plots a histogram
showing the sample pairs along the y-axis
and percent reproducible features per sample pair on
the x-axis.
}
\examples{
data <- matrix(rnorm(2400), nrow=200, ncol=12)
data_Marr <- Marr(object = data, pSamplepairs=0.75,
                  pFeatures=0.75, alpha=0.05)
MarrPlotSamplepairs(data_Marr)

}
